<?php

    require_once './vendor/autoload.php';

use Amp\Mysql\MysqlConfig;
use Amp\Mysql\MysqlConnectionPool;
use Amp\Promise;
use function Amp\async;

class Database {
    private $pool;

    public function __construct($host, $user, $password, $dbName) {
        $config = MysqlConfig::fromAuthority($host, $user, $password, $dbName);
        $this->pool = new MysqlConnectionPool($config);
    }

    public function createTable($table, $columns) {
        $columnDefs = array_map(function ($type, $col) { return "$col $type"; }, $columns, array_keys($columns));
        $columnDefs = implode(', ', $columnDefs);
        $query = "CREATE TABLE IF NOT EXISTS $table ($columnDefs)";
        return $this->executeQuery($query);
    }

    public function dropTable($table) {
        $query = "DROP TABLE IF EXISTS $table";
        return $this->executeQuery($query);
    }


    

    public function minValue($table, $column) {
        $query = "SELECT MIN($column) AS min_value FROM $table";
        return $this->executeQuery($query);
    }

    public function has($table, $conditions) {
        $conditionParts = [];
        foreach ($conditions as $key => $value) {
            $conditionParts[] = "$key = ?";
        }
        $whereClause = implode(' AND ', $conditionParts);
        $query = "SELECT EXISTS(SELECT 1 FROM $table WHERE $whereClause) AS has";
        return $this->executeQuery($query, array_values($conditions));
    }

    public function insert($table, array $data) {
        $columns = implode(', ', array_keys($data));
        $placeholders = implode(', ', array_fill(0, count($data), '?'));
        $query = "INSERT INTO $table ($columns) VALUES ($placeholders)";
        return $this->executeQuery($query, array_values($data));
    }

    public function update($table, array $data, array $conditions) {
        $setParts = [];
        foreach ($data as $key => $value) {
            $setParts[] = "$key = ?";
        }
        $setClause = implode(', ', $setParts);
        $conditionParts = [];
        foreach ($conditions as $key => $value) {
            $conditionParts[] = "$key = ?";
        }
        $whereClause = implode(' AND ', $conditionParts);
        $query = "UPDATE $table SET $setClause WHERE $whereClause";
        return $this->executeQuery($query, array_merge(array_values($data), array_values($conditions)));
    }

    public function delete($table, array $conditions) {
        $conditionParts = [];
        foreach ($conditions as $key => $value) {
            $conditionParts[] = "$key = ?";
        }
        $whereClause = implode(' AND ', $conditionParts);
        $query = "DELETE FROM $table WHERE $whereClause";
        return $this->executeQuery($query, array_values($conditions));
    }

    public function query($sql, $params = []) {
        return $this->executeQuery($sql, $params);
    }

    private function executeQuery($query, $params = []) {
        try {
            return $this->pool->execute($query, $params);
        } catch (\Throwable $e) {
            // Handle errors gracefully, perhaps log them and return null or false
            error_log("Database query failed: " . $e->getMessage());
            return null;
        }
    }

    public function beginTransaction() {
        return $this->pool->beginTransaction();
    }

    public function close() {
        $this->pool->close();
    }
}

// // Example usage:
// $db = new Database('localhost', 'root', '', 'test');
// // $db->createTable('tmp22', ['id' => 'INT AUTO_INCREMENT PRIMARY KEY', 'name' => 'VARCHAR(255)', 'age' => 'INT']);
// $db->insert('tmp2', ['id' =>21,'name' => 'ddddddddd', 'age' => 30]);
// // if (!$db->update('tmp2', ['age' => 90], ['name' => 'John Doe'])) {
// //     echo "Update failed!";
// // }
// // print_r($db->getMaxValue('tmp2', 'age'));
// // $db->dropTable('tmp2');



